@@ -22,6 +22,7 @@ use helper::{group_nodes_by_region, group_nodes_by_zone, is_aks, is_eks, NamesAn
 use kubeconfig::KubeconfigExt;
 
 mod crd;
+mod diagnose;
 mod helm;
 mod helper;
 mod kubeconfig;
@@ -134,8 +135,8 @@ impl Kubectl {
         })
     }
 
-    pub(crate) fn _has_context(&self, name: &str) -> bool {
-        self.kubeconfig.has_context(name)
+    pub(crate) fn context(&self) -> &str {
+        &self.context
     }
 
     pub(crate) fn cluster_name(&self) -> &v0::ClusterName {
@@ -174,20 +175,6 @@ impl Kubectl {
         Ok(pods.list(&lp).await?)
     }
 
-    async fn create_namespace(&self, namespace: &str) -> anyhow::Result<Namespace> {
-        let namespaces = self.api().await?;
-        let namespace = json::from_value(json::json!({
-            "apiVerion": "v1",
-            "kind": "Namespace",
-            "metadata": {
-                "name": namespace,
-            }
-        }))?;
-        let pp = self.post_params();
-        let namespace = namespaces.create(&pp, &namespace).await?;
-        Ok(namespace)
-    }
-
     async fn create_configmap(
         &self,
         name: &str,
@@ -347,13 +334,39 @@ impl Kubectl {
         .await
     }
 
-    pub(crate) async fn install_cluster_crd(&self, namespace: &str) -> anyhow::Result<()> {
-        let crd = self.install_crd().await?;
+    pub(crate) async fn ensure_statehub_cluster_is_not_installed(&self) -> anyhow::Result<()> {
+        if let Ok(cluster) = self.check_statehub_cluster_object().await {
+            anyhow::bail!(
+                "It looks like this cluster is already registered with Statehub under the name '{}'",
+                cluster.name()
+            );
+        }
+        Ok(())
+    }
+
+    pub(crate) async fn install_statehub_cluster(&self, namespace: &str) -> anyhow::Result<()> {
+        let crd = self.create_statehub_cluster_crd().await?;
         log::debug!("Created CRD: {:#?}", crd);
-        let name = self.cluster_name();
-        let cluster = self.create_statehub_cluster_crd(name, namespace).await?;
+        let cluster = self.create_statehub_cluster(namespace).await?;
         log::debug!("Created StatehubCluster: {:#?}", cluster);
 
+        let client = self.get_client().await?;
+        let statehub_kubectl = statehub_cluster_operator::cluster::Kubectl::new(client);
+        let namespace = statehub_kubectl.create_namespace(&cluster).await?;
+        log::debug!("Created namespace: {:#?}", namespace);
+        Ok(())
+    }
+
+    pub(crate) async fn uninstall_statehub_cluster(&self) -> anyhow::Result<()> {
+        let client = self.get_client().await?;
+        let cluster = self.check_statehub_cluster_object().await?;
+        let statehub_kubectl = statehub_cluster_operator::cluster::Kubectl::new(client);
+        statehub_kubectl.delete_namespace(&cluster).await?;
+
+        self.delete_statehub_cluster().await?;
+        log::debug!("Deleted StatehubCluster");
+        self.delete_statehub_cluster_crd().await?;
+        log::debug!("Deleted CRD");
         Ok(())
     }
 
@@ -370,23 +383,6 @@ impl Kubectl {
     }
 }
 
-pub(crate) async fn validate_namespace(namespace: &str) -> anyhow::Result<Namespace> {
-    let kube = Kubectl::new()?;
-    let existing = kube
-        .all_namespaces()
-        .await?
-        .into_iter()
-        .find(|ns| ns.name() == namespace);
-
-    if let Some(existing) = existing {
-        log::info!("Using existing namespace {}", existing.name());
-        Ok(existing)
-    } else {
-        log::info!("Creating new namespace {}", namespace);
-        kube.create_namespace(namespace).await
-    }
-}
-
 pub(crate) async fn list_namespaces() -> anyhow::Result<impl IntoIterator<Item = Namespace>> {
     Kubectl::new()?.all_namespaces().await
 }
