@@ -66,6 +66,9 @@ impl StateHub {
 
         let cluster_name = kubectl.cluster_name();
         let namespace = &register_cluster.namespace;
+
+        kubectl.ensure_statehub_cluster_is_not_installed().await?;
+
         let cluster = self
             .api
             .register_cluster(cluster_name, namespace, provider, locations)
@@ -79,7 +82,7 @@ impl StateHub {
             locations.show(),
         ))?;
 
-        k8s::validate_namespace(&register_cluster.namespace).await?;
+        kubectl.install_statehub_cluster(namespace).await?;
 
         self.setup_cluster_token_helper(&cluster, &register_cluster)
             .await?;
@@ -97,6 +100,22 @@ impl StateHub {
         cluster.print(&self.stdout, self.json)
     }
 
+    pub(super) async fn unregister_cluster_helper(
+        &self,
+        kubectl: k8s::Kubectl,
+    ) -> anyhow::Result<()> {
+        let name = kubectl.cluster_name();
+        self.relinquish_states_helper(name).await?;
+
+        self.api
+            .unregister_cluster(name)
+            .await
+            .print(&self.stdout, self.json)?;
+
+        kubectl.uninstall_statehub_cluster().await?;
+        Ok(())
+    }
+
     pub(super) async fn add_location_helper(
         &self,
         state: &v0::State,
